### $ANTLR 2.7.7 (2006-11-01): "group.g" -> "GroupParser.py"$
### import antlr and other modules ..
import sys
import antlr

version = sys.version.split()[0]
if version < '2.2.1':
    False = 0
if version < '2.3':
    True = not False
### header action >>> 
from ASTExpr import *
import stringtemplate3
import traceback
### header action <<< 
### preamble action>>>

### preamble action <<<

### import antlr.Token 
from antlr import Token
### >>>The Known Token Types <<<
SKIP                = antlr.SKIP
INVALID_TYPE        = antlr.INVALID_TYPE
EOF_TYPE            = antlr.EOF_TYPE
EOF                 = antlr.EOF
NULL_TREE_LOOKAHEAD = antlr.NULL_TREE_LOOKAHEAD
MIN_USER_TYPE       = antlr.MIN_USER_TYPE
LITERAL_group = 4
ID = 5
COLON = 6
LITERAL_implements = 7
COMMA = 8
SEMI = 9
AT = 10
DOT = 11
LPAREN = 12
RPAREN = 13
DEFINED_TO_BE = 14
STRING = 15
BIGSTRING = 16
ASSIGN = 17
ANONYMOUS_TEMPLATE = 18
LBRACK = 19
RBRACK = 20
LITERAL_default = 21
STAR = 22
PLUS = 23
OPTIONAL = 24
SL_COMMENT = 25
ML_COMMENT = 26
NL = 27
WS = 28


###/** Match a group of template definitions beginning
### *  with a group name declaration.  Templates are enclosed
### *  in double-quotes or <<...>> quotes for multi-line templates.
### *  Template names have arg lists that indicate the cardinality
### *  of the attribute: present, optional, zero-or-more, one-or-more.
### *  Here is a sample group file:
###
###        group nfa;
###
###        // an NFA has edges and states
###        nfa(states,edges) ::= <<
###        digraph NFA {
###        rankdir=LR;
###        <states; separator="\\n">
###        <edges; separator="\\n">
###        }
###        >>
###
###        state(name) ::= "node [shape = circle]; <name>;"
###
### */
class Parser(antlr.LLkParser):
    ### user action >>>
    def reportError(self, e):
       if self.group_:
           self.group_.error("template group parse error", e)
       else:
           sys.stderr.write("template group parse error: " + str(e) + '\n')
           traceback.print_exc()
    ### user action <<<
    
    def __init__(self, *args, **kwargs):
        antlr.LLkParser.__init__(self, *args, **kwargs)
        self.tokenNames = _tokenNames
        ### __init__ header action >>> 
        self.group_ = None
        ### __init__ header action <<< 
        
    def group(self,
        g
    ):    
        
        name = None
        s = None
        i = None
        i2 = None
        self.group_ = g
        try:      ## for error handling
            pass
            self.match(LITERAL_group)
            name = self.LT(1)
            self.match(ID)
            g.name = name.getText()
            la1 = self.LA(1)
            if False:
                pass
            elif la1 and la1 in [COLON]:
                pass
                self.match(COLON)
                s = self.LT(1)
                self.match(ID)
                g.superGroup = s.getText()
            elif la1 and la1 in [LITERAL_implements,SEMI]:
                pass
            else:
                    raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                
            la1 = self.LA(1)
            if False:
                pass
            elif la1 and la1 in [LITERAL_implements]:
                pass
                self.match(LITERAL_implements)
                i = self.LT(1)
                self.match(ID)
                g.implementInterface(i.getText())
                while True:
                    if (self.LA(1)==COMMA):
                        pass
                        self.match(COMMA)
                        i2 = self.LT(1)
                        self.match(ID)
                        g.implementInterface(i2.getText())
                    else:
                        break
                    
            elif la1 and la1 in [SEMI]:
                pass
            else:
                    raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                
            self.match(SEMI)
            while True:
                if (self.LA(1)==ID or self.LA(1)==AT) and (self.LA(2)==ID or self.LA(2)==LPAREN or self.LA(2)==DEFINED_TO_BE) and (self.LA(3)==ID or self.LA(3)==DOT or self.LA(3)==RPAREN):
                    pass
                    self.template(g)
                elif (self.LA(1)==ID) and (self.LA(2)==DEFINED_TO_BE) and (self.LA(3)==LBRACK):
                    pass
                    self.mapdef(g)
                else:
                    break
                
            self.match(EOF_TYPE)
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_0)
        
    
    def template(self,
        g
    ):    
        
        scope = None
        region = None
        name = None
        t = None
        bt = None
        alias = None
        target = None
        formalArgs = {}
        st = None
        ignore = False
        templateName = None
        line = self.LT(1).getLine()
        try:      ## for error handling
            if (self.LA(1)==ID or self.LA(1)==AT) and (self.LA(2)==ID or self.LA(2)==LPAREN):
                pass
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in [AT]:
                    pass
                    self.match(AT)
                    scope = self.LT(1)
                    self.match(ID)
                    self.match(DOT)
                    region = self.LT(1)
                    self.match(ID)
                    templateName = g.getMangledRegionName(scope.getText(),region.getText())
                    if g.isDefinedInThisGroup(templateName):
                       g.error("group "+g.name+" line "+str(line)+": redefinition of template region: @"+
                           scope.getText()+"."+region.getText())
                       st = stringtemplate3.StringTemplate() # create bogus template to fill in
                    
                    else:
                       err = False
                       # @template.region() ::= "..."
                       scopeST = g.lookupTemplate(scope.getText())
                       if scopeST is None:
                           g.error("group "+g.name+" line "+str(line)+": reference to region within undefined template: "+
                               scope.getText())
                           err = True
                    
                       if not scopeST.containsRegionName(region.getText()):
                           g.error("group "+g.name+" line "+str(line)+": template "+scope.getText()+" has no region called "+
                               region.getText())
                           err = True
                    
                       if err:
                           st = stringtemplate3.StringTemplate()
                    
                       else:
                           st = g.defineRegionTemplate(
                               scope.getText(),
                               region.getText(),
                               None,
                               stringtemplate3.REGION_EXPLICIT
                           )
                elif la1 and la1 in [ID]:
                    pass
                    name = self.LT(1)
                    self.match(ID)
                    templateName = name.getText()
                    if g.isDefinedInThisGroup(templateName):
                       g.error("redefinition of template: " + templateName)
                       # create bogus template to fill in
                       st = stringtemplate3.StringTemplate()
                    else:
                       st = g.defineTemplate(templateName, None)
                else:
                        raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                    
                if st is not None:
                   st.groupFileLine = line
                self.match(LPAREN)
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in [ID]:
                    pass
                    self.args(st)
                elif la1 and la1 in [RPAREN]:
                    pass
                    st.defineEmptyFormalArgumentList()
                else:
                        raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                    
                self.match(RPAREN)
                self.match(DEFINED_TO_BE)
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in [STRING]:
                    pass
                    t = self.LT(1)
                    self.match(STRING)
                    st.template = t.getText()
                elif la1 and la1 in [BIGSTRING]:
                    pass
                    bt = self.LT(1)
                    self.match(BIGSTRING)
                    st.template = bt.getText()
                else:
                        raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                    
            elif (self.LA(1)==ID) and (self.LA(2)==DEFINED_TO_BE):
                pass
                alias = self.LT(1)
                self.match(ID)
                self.match(DEFINED_TO_BE)
                target = self.LT(1)
                self.match(ID)
                g.defineTemplateAlias(alias.getText(), target.getText())
            else:
                raise antlr.NoViableAltException(self.LT(1), self.getFilename())
            
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_1)
        
    
    def mapdef(self,
        g
    ):    
        
        name = None
        m = None
        try:      ## for error handling
            pass
            name = self.LT(1)
            self.match(ID)
            self.match(DEFINED_TO_BE)
            m=self.map()
            if g.getMap(name.getText()):
               g.error("redefinition of map: " + name.getText())
            elif g.isDefinedInThisGroup(name.getText()):
               g.error("redefinition of template as map: " + name.getText())
            else:
               g.defineMap(name.getText(), m)
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_1)
        
    
    def args(self,
        st
    ):    
        
        try:      ## for error handling
            pass
            self.arg(st)
            while True:
                if (self.LA(1)==COMMA):
                    pass
                    self.match(COMMA)
                    self.arg(st)
                else:
                    break
                
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_2)
        
    
    def arg(self,
        st
    ):    
        
        name = None
        s = None
        bs = None
        defaultValue = None
        try:      ## for error handling
            pass
            name = self.LT(1)
            self.match(ID)
            if (self.LA(1)==ASSIGN) and (self.LA(2)==STRING):
                pass
                self.match(ASSIGN)
                s = self.LT(1)
                self.match(STRING)
                defaultValue = stringtemplate3.StringTemplate(
                     template="$_val_$"
                     )
                defaultValue["_val_"] = s.getText()
                defaultValue.defineFormalArgument("_val_")
                defaultValue.name = ("<" + st.name + "'s arg " +
                                    name.getText() +
                                    " default value subtemplate>")
            elif (self.LA(1)==ASSIGN) and (self.LA(2)==ANONYMOUS_TEMPLATE):
                pass
                self.match(ASSIGN)
                bs = self.LT(1)
                self.match(ANONYMOUS_TEMPLATE)
                defaultValue = stringtemplate3.StringTemplate(
                     group=st.group,
                     template=bs.getText()
                     )
                defaultValue.name = ("<" + st.name + "'s arg " +
                                    name.getText() +
                                    " default value subtemplate>")
            elif (self.LA(1)==COMMA or self.LA(1)==RPAREN):
                pass
            else:
                raise antlr.NoViableAltException(self.LT(1), self.getFilename())
            
            st.defineFormalArgument(name.getText(), defaultValue)
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_3)
        
    
    def map(self):    
        mapping = {}
        
        try:      ## for error handling
            pass
            self.match(LBRACK)
            self.mapPairs(mapping)
            self.match(RBRACK)
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_1)
        
        return mapping
    
    def mapPairs(self,
        mapping
    ):    
        
        try:      ## for error handling
            la1 = self.LA(1)
            if False:
                pass
            elif la1 and la1 in [STRING]:
                pass
                self.keyValuePair(mapping)
                while True:
                    if (self.LA(1)==COMMA) and (self.LA(2)==STRING):
                        pass
                        self.match(COMMA)
                        self.keyValuePair(mapping)
                    else:
                        break
                    
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in [COMMA]:
                    pass
                    self.match(COMMA)
                    self.defaultValuePair(mapping)
                elif la1 and la1 in [RBRACK]:
                    pass
                else:
                        raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                    
            elif la1 and la1 in [LITERAL_default]:
                pass
                self.defaultValuePair(mapping)
            else:
                    raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_4)
        
    
    def keyValuePair(self,
        mapping
    ):    
        
        key = None
        try:      ## for error handling
            pass
            key = self.LT(1)
            self.match(STRING)
            self.match(COLON)
            v=self.keyValue()
            mapping[key.getText()] = v
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_5)
        
    
    def defaultValuePair(self,
        mapping
    ):    
        
        try:      ## for error handling
            pass
            self.match(LITERAL_default)
            self.match(COLON)
            v=self.keyValue()
            mapping[stringtemplate3.language.ASTExpr.DEFAULT_MAP_VALUE_NAME] = v
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_4)
        
    
    def keyValue(self):    
        value = None
        
        s1 = None
        s2 = None
        k = None
        try:      ## for error handling
            la1 = self.LA(1)
            if False:
                pass
            elif la1 and la1 in [STRING]:
                pass
                s1 = self.LT(1)
                self.match(STRING)
                value = stringtemplate3.StringTemplate(
                   group=self.group_, template=s1.getText()
                   )
            elif la1 and la1 in [BIGSTRING]:
                pass
                s2 = self.LT(1)
                self.match(BIGSTRING)
                value = stringtemplate3.StringTemplate(
                   group=self.group_,
                   template=s2.getText()
                   )
            elif la1 and la1 in [ID]:
                pass
                k = self.LT(1)
                self.match(ID)
                if not  k.getText() == "key" :
                    raise antlr.SemanticException(" k.getText() == \"key\" ")
                value = stringtemplate3.language.ASTExpr.MAP_KEY_VALUE
            elif la1 and la1 in [COMMA,RBRACK]:
                pass
            else:
                    raise antlr.NoViableAltException(self.LT(1), self.getFilename())
                
        
        except antlr.RecognitionException, ex:
            self.reportError(ex)
            self.consume()
            self.consumeUntil(_tokenSet_5)
        
        return value
    

_tokenNames = [
    "<0>", 
    "EOF", 
    "<2>", 
    "NULL_TREE_LOOKAHEAD", 
    "\"group\"", 
    "ID", 
    "COLON", 
    "\"implements\"", 
    "COMMA", 
    "SEMI", 
    "AT", 
    "DOT", 
    "LPAREN", 
    "RPAREN", 
    "DEFINED_TO_BE", 
    "STRING", 
    "BIGSTRING", 
    "ASSIGN", 
    "ANONYMOUS_TEMPLATE", 
    "LBRACK", 
    "RBRACK", 
    "\"default\"", 
    "STAR", 
    "PLUS", 
    "OPTIONAL", 
    "SL_COMMENT", 
    "ML_COMMENT", 
    "NL", 
    "WS"
]
    

### generate bit set
def mk_tokenSet_0(): 
    ### var1
    data = [ 2L, 0L]
    return data
_tokenSet_0 = antlr.BitSet(mk_tokenSet_0())

### generate bit set
def mk_tokenSet_1(): 
    ### var1
    data = [ 1058L, 0L]
    return data
_tokenSet_1 = antlr.BitSet(mk_tokenSet_1())

### generate bit set
def mk_tokenSet_2(): 
    ### var1
    data = [ 8192L, 0L]
    return data
_tokenSet_2 = antlr.BitSet(mk_tokenSet_2())

### generate bit set
def mk_tokenSet_3(): 
    ### var1
    data = [ 8448L, 0L]
    return data
_tokenSet_3 = antlr.BitSet(mk_tokenSet_3())

### generate bit set
def mk_tokenSet_4(): 
    ### var1
    data = [ 1048576L, 0L]
    return data
_tokenSet_4 = antlr.BitSet(mk_tokenSet_4())

### generate bit set
def mk_tokenSet_5(): 
    ### var1
    data = [ 1048832L, 0L]
    return data
_tokenSet_5 = antlr.BitSet(mk_tokenSet_5())
    




from stringtemplate3.language.ASTExpr import *
from stringtemplate3.language.ActionEvaluator import *
from stringtemplate3.language.ActionLexer import *
from stringtemplate3.language.ActionParser import *
from stringtemplate3.language.AngleBracketTemplateLexer import *
from stringtemplate3.language.ChunkToken import *
from stringtemplate3.language.ConditionalExpr import *
from stringtemplate3.language.DefaultTemplateLexer import *
from stringtemplate3.language.Expr import *
from stringtemplate3.language.FormalArgument import *
from stringtemplate3.language.GroupLexer import *
from stringtemplate3.language.GroupParser import *
from stringtemplate3.language.NewlineRef import *
from stringtemplate3.language.StringRef import *
from stringtemplate3.language.StringTemplateAST import *
from stringtemplate3.language.TemplateParser import *




__all__ = ['language']

__version__ = "3.1"

## track probable issues like setting attribute that is not referenced.
#  Set to true to make StringTemplate check your work as it evaluates
#  templates.  Problems are sent to error listener.  Currently warns when
#  you set attributes that are not used.
lintMode = False

from stringtemplate3.errors import *
from stringtemplate3.writers import *
from stringtemplate3.templates import *
from stringtemplate3.groups import *
from stringtemplate3.interfaces import *
from stringtemplate3.grouploaders import *






import antlr

from stringtemplate3.language import ASTExpr
from stringtemplate3.language import ActionEvaluator
from stringtemplate3.utils import deprecated


class ElseIfClauseData(object):
    def __init__(self, expr, st):
        self.expr = expr
        self.st = st


class ConditionalExpr(ASTExpr):
    """A conditional reference to an embedded subtemplate."""

    def __init__(self, enclosingTemplate, tree):
        super(ConditionalExpr, self).__init__(enclosingTemplate, tree, None)
        self.subtemplate = None
        self.elseIfSubtemplates = None
        self.elseSubtemplate = None


    @deprecated
    def setSubtemplate(self, subtemplate):
        self.subtemplate = subtemplate

    @deprecated
    def getSubtemplate(self):
        return self.subtemplate


    @deprecated
    def setElseSubtemplate(self, elseSubtemplate):
        self.elseSubtemplate = elseSubtemplate

    @deprecated
    def getElseSubtemplate(self):
        return self.elseSubtemplate


    def addElseIfSubtemplate(self, conditionalTree, subtemplate):
        if self.elseIfSubtemplates is None:
            self.elseIfSubtemplates = []

        d = ElseIfClauseData(conditionalTree, subtemplate)
        self.elseIfSubtemplates.append(d)


    def write(self, this, out):
        """
        To write out the value of a condition expr, invoke the evaluator in
        eval.g to walk the condition tree computing the boolean value.
        If result is true, then write subtemplate.
        """

        if self.exprTree is None or this is None or out is None:
            return 0
        
        evaluator = ActionEvaluator.Walker()
        evaluator.initialize(this, self, out)
        n = 0
        try:
            testedTrue = False
            # get conditional from tree and compute result
            cond = self.exprTree.getFirstChild()

            # eval and write out tree. In case the condition refers to an
            # undefined attribute, we catch the KeyError exception and proceed
            # with a False value.
            try:
                includeSubtemplate = evaluator.ifCondition(cond)
            except KeyError, ke:
                includeSubtemplate = False

            if includeSubtemplate:
                n = self.writeSubTemplate(this, out, self.subtemplate)
                testedTrue = True

            elif ( self.elseIfSubtemplates is not None and
                   len(self.elseIfSubtemplates) > 0 ):
                for elseIfClause in self.elseIfSubtemplates:
                    try:
                        includeSubtemplate = evaluator.ifCondition(elseIfClause.expr.exprTree)
                    except KeyError, ke:
                        includeSubtemplate = False
                    if includeSubtemplate:
                        n = self.writeSubTemplate(this, out, elseIfClause.st)
                        testedTrue = True
                        break

            if not testedTrue and self.elseSubtemplate is not None:
                # evaluate ELSE clause if present and IF/ELSEIF conditions
                # failed
                n = self.writeSubTemplate(this, out, self.elseSubtemplate)
                        
        except antlr.RecognitionException, re:
            this.error(
                "can't evaluate tree: " + self.exprTree.toStringList(), re
                )

        return n


    def writeSubTemplate(self, this, out, subtemplate):
        # To evaluate the IF chunk, make a new instance whose enclosingInstance
        # points at 'this' so get attribute works. Otherwise, enclosingInstance
        # points at the template used to make the precompiled code.  We need a
        # new template instance every time we exec this chunk to get the new
        # "enclosing instance" pointer.

        s = subtemplate.getInstanceOf()
        s.enclosingInstance = this
        # make sure we evaluate in context of enclosing template's
        # group so polymorphism works. :)
        s.group = this.group
        s.nativeGroup = this.nativeGroup
        return s.write(out)



#!/usr/bin/env python

from distutils.core import setup


setup(
    name="stringtemplate3",
    version="3.1",
    description="A powerful template engine with strict model-view separation",
    long_description="""
    ST (StringTemplate) is a template engine for generating source code, web
    pages, emails, or any other formatted text output. ST is particularly
    good at multi-targeted code generators, multiple site skins, and
    internationalization/localization. It evolved over years of effort
    developing jGuru.com. ST also generates this website and powers the
    ANTLR v3 code generator. Its distinguishing characteristic is that it
    strictly enforces model-view separation unlike other engines.
    """,
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Text Processing',
    ],
    author="Terence Parr / Marq Kole / Benjamin Niemann",
    author_email="parrt AT antlr.org / marq.kole AT philips.com / pink@odahoda.de",
    maintainer="Benjamin Niemann",
    maintainer_email="pink@odahoda.de",
    url="http://www.stringtemplate.org/",
    license="BSD",
    platform="any",
    packages=['stringtemplate3', 'stringtemplate3.language'],
    )




from StringIO import StringIO

## Given a list of lists, return the combined elements one by one.
#
class CatList(object):

    def __init__(self, lists):
	## List of lists to cat together 
	#
        self._lists = lists

    def __len__(self):
        k = 0
        for list_ in self._lists:
            k += len(list_)
        return k

    def lists(self):
        for list_ in self._lists:
            for item in list_:
                yield item

    def __iter__(self):
        for list_ in self._lists:
            for item in list_:
                yield item

        
    ## The result of asking for the string of a CatList is the list of
    #  items and so this is just the cat'd list of both items.  This
    #  is destructive in that the iterator cursors have moved to the end
    #  after printing.
    def __str__(self):
        buf = StringIO()
        #buf.write('[')
        k = len(self)
        for item in self.lists():
            buf.write(str(item))
            k -= 1
            #if k:
            #    buf.write(', ')
        #buf.write(']')

        return buf.getvalue()

    __repr__ = __str__




# [The "BSD licence"]
# Copyright (c) 2003-2006 Terence Parr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import sys
import os
import traceback
import codecs

from stringtemplate3.utils import deprecated
from stringtemplate3.groups import StringTemplateGroup
from stringtemplate3.interfaces import StringTemplateGroupInterface
from stringtemplate3.language import AngleBracketTemplateLexer


class StringTemplateGroupLoader(object):
    """
    When group files derive from another group, we have to know how to
    load that group and its supergroups. This interface also knows how
    to load interfaces
    """

    def loadGroup(self, groupName, superGroup=None, lexer=None):
        """
        Load the group called groupName from somewhere.  Return null
        if no group is found.
        Groups with region definitions must know their supergroup to find
        templates during parsing.
        Specify the template lexer to use for parsing templates.  If null,
        it assumes angle brackets <...>.
        """
        
        raise NotImplementedError


    def loadInterface(self, interfaceName):
        """
        Load the interface called interfaceName from somewhere.  Return null
        if no interface is found.
        """

        raise NotImplementedError
    

class PathGroupLoader(StringTemplateGroupLoader):
    """
    A brain dead loader that looks only in the directory(ies) you
    specify in the ctor.
    You may specify the char encoding.
    """
    
    def __init__(self, dir=None, errors=None):
        """
        Pass a single dir or multiple dirs separated by colons from which
        to load groups/interfaces.
        """
        
        StringTemplateGroupLoader.__init__(self)

        ## List of ':' separated dirs to pull groups from
        self.dirs = dir.split(':')
        self.errors = errors

        ## How are the files encoded (ascii, UTF8, ...)?
        #  You might want to read UTF8 for example on an ascii machine.
        self.fileCharEncoding = sys.getdefaultencoding()
        

    def loadGroup(self, groupName, superGroup=None, lexer=None):
        if lexer is None:
            lexer = AngleBracketTemplateLexer.Lexer
        try:
            fr = self.locate(groupName+".stg")
            if fr is None:
                self.error("no such group file "+groupName+".stg")
                return None

            try:
                return StringTemplateGroup(
                    file=fr,
                    lexer=lexer,
                    errors=self.errors,
                    superGroup=superGroup
                    )
            finally:
                fr.close()

        except IOError, ioe:
            self.error("can't load group "+groupName, ioe)

        return None


    def loadInterface(self, interfaceName):
        try:
            fr = self.locate(interfaceName+".sti")
            if fr is None:
                self.error("no such interface file "+interfaceName+".sti")
                return None

            try:
                return StringTemplateGroupInterface(fr, self.errors)

            finally:
                fr.close()
                
        except (IOError, OSError), ioe:
            self.error("can't load interface "+interfaceName, ioe)

        return None


    def locate(self, name):
        """Look in each directory for the file called 'name'."""
        
        for dir in self.dirs:
            path = os.path.join(dir, name)
            if os.path.isfile(path):
                fr = open(path, 'r')
                # FIXME: something breaks, when stream return unicode
                if self.fileCharEncoding is not None:
                    fr = codecs.getreader(self.fileCharEncoding)(fr)
                return fr

        return None
    

    @deprecated
    def getFileCharEncoding(self):
        return self.fileCharEncoding

    @deprecated
    def setFileCharEncoding(self, fileCharEncoding):
        self.fileCharEncoding = fileCharEncoding


    def error(self, msg, exc=None):
        if self.errors is not None:
            self.errors.error(msg, exc)
            
        else:
            sys.stderr.write("StringTemplate: "+msg+"\n")
            if exc is not None:
                traceback.print_exc()


class CommonGroupLoader(PathGroupLoader):
    """
    Subclass o PathGroupLoader that also works, if the package is
    packaged in a zip file.
    FIXME: this is not yet implemented, behaviour is identical to
    PathGroupLoader!
    """

    # FIXME: this needs to be overridden!
    def locate(self, name):
        """Look in each directory for the file called 'name'."""

        return PathGroupLoader.locate(self, name)





### $ANTLR 2.7.7 (2006-11-01): "action.g" -> "ActionLexer.py"$
### import antlr and other modules ..
import sys
import antlr

version = sys.version.split()[0]
if version < '2.2.1':
    False = 0
if version < '2.3':
    True = not False
### header action >>> 
from stringtemplate3.language.StringTemplateToken import StringTemplateToken
import stringtemplate3
### header action <<< 
### preamble action >>> 

### preamble action <<< 
### >>>The Literals<<<
literals = {}
literals[u"super"] = 32
literals[u"if"] = 8
literals[u"first"] = 26
literals[u"last"] = 28
literals[u"rest"] = 27
literals[u"trunc"] = 31
literals[u"strip"] = 30
literals[u"length"] = 29
literals[u"elseif"] = 18


### import antlr.Token 
from antlr import Token
### >>>The Known Token Types <<<
SKIP                = antlr.SKIP
INVALID_TYPE        = antlr.INVALID_TYPE
EOF_TYPE            = antlr.EOF_TYPE
EOF                 = antlr.EOF
NULL_TREE_LOOKAHEAD = antlr.NULL_TREE_LOOKAHEAD
MIN_USER_TYPE       = antlr.MIN_USER_TYPE
APPLY = 4
MULTI_APPLY = 5
ARGS = 6
INCLUDE = 7
CONDITIONAL = 8
VALUE = 9
TEMPLATE = 10
FUNCTION = 11
SINGLEVALUEARG = 12
LIST = 13
NOTHING = 14
SEMI = 15
LPAREN = 16
RPAREN = 17
LITERAL_elseif = 18
COMMA = 19
ID = 20
ASSIGN = 21
COLON = 22
NOT = 23
PLUS = 24
DOT = 25
LITERAL_first = 26
LITERAL_rest = 27
LITERAL_last = 28
LITERAL_length = 29
LITERAL_strip = 30
LITERAL_trunc = 31
LITERAL_super = 32
ANONYMOUS_TEMPLATE = 33
STRING = 34
INT = 35
LBRACK = 36
RBRACK = 37
DOTDOTDOT = 38
TEMPLATE_ARGS = 39
NESTED_ANONYMOUS_TEMPLATE = 40
ESC_CHAR = 41
WS = 42
WS_CHAR = 43

class Lexer(antlr.CharScanner) :
    ### user action >>>
    ### user action <<<
    def __init__(self, *argv, **kwargs) :
        antlr.CharScanner.__init__(self, *argv, **kwargs)
        self.caseSensitiveLiterals = True
        self.setCaseSensitive(True)
        self.literals = literals
    
    def nextToken(self):
        while True:
            try: ### try again ..
                while True:
                    _token = None
                    _ttype = INVALID_TYPE
                    self.resetText()
                    try: ## for char stream error handling
                        try: ##for lexical error handling
                            la1 = self.LA(1)
                            if False:
                                pass
                            elif la1 and la1 in u'ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz':
                                pass
                                self.mID(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'0123456789':
                                pass
                                self.mINT(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'"':
                                pass
                                self.mSTRING(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'{':
                                pass
                                self.mANONYMOUS_TEMPLATE(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'[':
                                pass
                                self.mLBRACK(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u']':
                                pass
                                self.mRBRACK(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'(':
                                pass
                                self.mLPAREN(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u')':
                                pass
                                self.mRPAREN(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u',':
                                pass
                                self.mCOMMA(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'=':
                                pass
                                self.mASSIGN(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u':':
                                pass
                                self.mCOLON(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'+':
                                pass
                                self.mPLUS(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u';':
                                pass
                                self.mSEMI(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'!':
                                pass
                                self.mNOT(True)
                                theRetToken = self._returnToken
                            elif la1 and la1 in u'\t\n\r ':
                                pass
                                self.mWS(True)
                                theRetToken = self._returnToken
                            else:
                                if (self.LA(1)==u'.') and (self.LA(2)==u'.'):
                                    pass
                                    self.mDOTDOTDOT(True)
                                    theRetToken = self._returnToken
                                elif (self.LA(1)==u'.') and (True):
                                    pass
                                    self.mDOT(True)
                                    theRetToken = self._returnToken
                                else:
                                    self.default(self.LA(1))
                                
                            if not self._returnToken:
                                raise antlr.TryAgain ### found SKIP token
                            ### return token to caller
                            return self._returnToken
                        ### handle lexical errors ....
                        except antlr.RecognitionException, e:
                            raise antlr.TokenStreamRecognitionException(e)
                    ### handle char stream errors ...
                    except antlr.CharStreamException,cse:
                        if isinstance(cse, antlr.CharStreamIOException):
                            raise antlr.TokenStreamIOException(cse.io)
                        else:
                            raise antlr.TokenStreamException(str(cse))
            except antlr.TryAgain:
                pass
        
    def mID(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = ID
        _saveIndex = 0
        pass
        la1 = self.LA(1)
        if False:
            pass
        elif la1 and la1 in u'abcdefghijklmnopqrstuvwxyz':
            pass
            self.matchRange(u'a', u'z')
        elif la1 and la1 in u'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
            pass
            self.matchRange(u'A', u'Z')
        elif la1 and la1 in u'_':
            pass
            self.match('_')
        else:
                self.raise_NoViableAlt(self.LA(1))
            
        while True:
            la1 = self.LA(1)
            if False:
                pass
            elif la1 and la1 in u'abcdefghijklmnopqrstuvwxyz':
                pass
                self.matchRange(u'a', u'z')
            elif la1 and la1 in u'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
                pass
                self.matchRange(u'A', u'Z')
            elif la1 and la1 in u'0123456789':
                pass
                self.matchRange(u'0', u'9')
            elif la1 and la1 in u'_':
                pass
                self.match('_')
            elif la1 and la1 in u'/':
                pass
                self.match('/')
            else:
                    break
                
        ### option { testLiterals=true } 
        _ttype = self.testLiteralsTable(_ttype)
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mINT(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = INT
        _saveIndex = 0
        pass
        _cnt63= 0
        while True:
            if ((self.LA(1) >= u'0' and self.LA(1) <= u'9')):
                pass
                self.matchRange(u'0', u'9')
            else:
                break
            
            _cnt63 += 1
        if _cnt63 < 1:
            self.raise_NoViableAlt(self.LA(1))
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mSTRING(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = STRING
        _saveIndex = 0
        pass
        _saveIndex = self.text.length()
        self.match('"')
        self.text.setLength(_saveIndex)
        while True:
            if (self.LA(1)==u'\\'):
                pass
                self.mESC_CHAR(False, True)
            elif (_tokenSet_0.member(self.LA(1))):
                pass
                self.matchNot('"')
            else:
                break
            
        _saveIndex = self.text.length()
        self.match('"')
        self.text.setLength(_saveIndex)
        self.set_return_token(_createToken, _token, _ttype, _begin)
    

    ###/** Match escape sequences, optionally translating them for strings, but not
    ### *  for templates.  Do \} only when in {...} templates.
    ### */
    def mESC_CHAR(self, _createToken,
        doEscape
    ):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = ESC_CHAR
        _saveIndex = 0
        c = '\0'
        pass
        self.match('\\')
        if (self.LA(1)==u'n') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            self.match('n')
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append("\n")
        elif (self.LA(1)==u'r') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            self.match('r')
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append("\r")
        elif (self.LA(1)==u't') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            self.match('t')
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append("\t")
        elif (self.LA(1)==u'b') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            self.match('b')
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append("\b")
        elif (self.LA(1)==u'f') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            self.match('f')
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append("\f")
        elif ((self.LA(1) >= u'\u0003' and self.LA(1) <= u'\ufffe')) and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
            pass
            c = self.LA(1)
            self.matchNot(antlr.EOF_CHAR)
            if not self.inputState.guessing:
                if doEscape: self.text.setLength(_begin) ; self.text.append(str(c))
        else:
            self.raise_NoViableAlt(self.LA(1))
        
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mANONYMOUS_TEMPLATE(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = ANONYMOUS_TEMPLATE
        _saveIndex = 0
        args = None
        t = None
        pass
        _saveIndex = self.text.length()
        self.match('{')
        self.text.setLength(_saveIndex)
        synPredMatched70 = False
        if (_tokenSet_1.member(self.LA(1))) and (_tokenSet_2.member(self.LA(2))):
            _m70 = self.mark()
            synPredMatched70 = True
            self.inputState.guessing += 1
            try:
                pass
                self.mTEMPLATE_ARGS(False)
            except antlr.RecognitionException, pe:
                synPredMatched70 = False
            self.rewind(_m70)
            self.inputState.guessing -= 1
        if synPredMatched70:
            pass
            args=self.mTEMPLATE_ARGS(False)
            if (_tokenSet_3.member(self.LA(1))) and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
                pass
                _saveIndex = self.text.length()
                self.mWS_CHAR(False)
                self.text.setLength(_saveIndex)
            elif ((self.LA(1) >= u'\u0003' and self.LA(1) <= u'\ufffe')) and (True):
                pass
            else:
                self.raise_NoViableAlt(self.LA(1))
            
            if not self.inputState.guessing:
                # create a special token to track args
                t = StringTemplateToken(ANONYMOUS_TEMPLATE, self.text.getString(_begin), args)
                _token = t
        elif ((self.LA(1) >= u'\u0003' and self.LA(1) <= u'\ufffe')) and (True):
            pass
        else:
            self.raise_NoViableAlt(self.LA(1))
        
        while True:
            if (self.LA(1)==u'\\') and (self.LA(2)==u'{'):
                pass
                _saveIndex = self.text.length()
                self.match('\\')
                self.text.setLength(_saveIndex)
                self.match('{')
            elif (self.LA(1)==u'\\') and (self.LA(2)==u'}'):
                pass
                _saveIndex = self.text.length()
                self.match('\\')
                self.text.setLength(_saveIndex)
                self.match('}')
            elif (self.LA(1)==u'\\') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
                pass
                self.mESC_CHAR(False, False)
            elif (self.LA(1)==u'{'):
                pass
                self.mNESTED_ANONYMOUS_TEMPLATE(False)
            elif (_tokenSet_4.member(self.LA(1))):
                pass
                self.matchNot('}')
            else:
                break
            
        if not self.inputState.guessing:
            if t:
               t.setText(self.text.getString(_begin))
        _saveIndex = self.text.length()
        self.match('}')
        self.text.setLength(_saveIndex)
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mTEMPLATE_ARGS(self, _createToken):    
        args=[]
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = TEMPLATE_ARGS
        _saveIndex = 0
        a = None
        a2 = None
        pass
        la1 = self.LA(1)
        if False:
            pass
        elif la1 and la1 in u'\t\n\r ':
            pass
            _saveIndex = self.text.length()
            self.mWS_CHAR(False)
            self.text.setLength(_saveIndex)
        elif la1 and la1 in u'ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz':
            pass
        else:
                self.raise_NoViableAlt(self.LA(1))
            
        _saveIndex = self.text.length()
        self.mID(True)
        self.text.setLength(_saveIndex)
        a = self._returnToken
        if not self.inputState.guessing:
            args.append(a.getText())
        while True:
            if (_tokenSet_5.member(self.LA(1))) and (_tokenSet_6.member(self.LA(2))):
                pass
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in u'\t\n\r ':
                    pass
                    _saveIndex = self.text.length()
                    self.mWS_CHAR(False)
                    self.text.setLength(_saveIndex)
                elif la1 and la1 in u',':
                    pass
                else:
                        self.raise_NoViableAlt(self.LA(1))
                    
                _saveIndex = self.text.length()
                self.match(',')
                self.text.setLength(_saveIndex)
                la1 = self.LA(1)
                if False:
                    pass
                elif la1 and la1 in u'\t\n\r ':
                    pass
                    _saveIndex = self.text.length()
                    self.mWS_CHAR(False)
                    self.text.setLength(_saveIndex)
                elif la1 and la1 in u'ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz':
                    pass
                else:
                        self.raise_NoViableAlt(self.LA(1))
                    
                _saveIndex = self.text.length()
                self.mID(True)
                self.text.setLength(_saveIndex)
                a2 = self._returnToken
                if not self.inputState.guessing:
                    args.append(a2.getText())
            else:
                break
            
        la1 = self.LA(1)
        if False:
            pass
        elif la1 and la1 in u'\t\n\r ':
            pass
            _saveIndex = self.text.length()
            self.mWS_CHAR(False)
            self.text.setLength(_saveIndex)
        elif la1 and la1 in u'|':
            pass
        else:
                self.raise_NoViableAlt(self.LA(1))
            
        _saveIndex = self.text.length()
        self.match('|')
        self.text.setLength(_saveIndex)
        self.set_return_token(_createToken, _token, _ttype, _begin)
        return args
    
    def mWS_CHAR(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = WS_CHAR
        _saveIndex = 0
        la1 = self.LA(1)
        if False:
            pass
        elif la1 and la1 in u' ':
            pass
            self.match(' ')
        elif la1 and la1 in u'\t':
            pass
            self.match('\t')
        elif la1 and la1 in u'\n\r':
            pass
            if (self.LA(1)==u'\r') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
                pass
                self.match('\r')
            elif (self.LA(1)==u'\r') and (self.LA(2)==u'\n'):
                pass
                self.match('\r')
                self.match('\n')
            elif (self.LA(1)==u'\n'):
                pass
                self.match('\n')
            else:
                self.raise_NoViableAlt(self.LA(1))
            
            if not self.inputState.guessing:
                self.newline()
        else:
                self.raise_NoViableAlt(self.LA(1))
            
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mNESTED_ANONYMOUS_TEMPLATE(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = NESTED_ANONYMOUS_TEMPLATE
        _saveIndex = 0
        pass
        self.match('{')
        while True:
            if (self.LA(1)==u'\\') and (self.LA(2)==u'{'):
                pass
                _saveIndex = self.text.length()
                self.match('\\')
                self.text.setLength(_saveIndex)
                self.match('{')
            elif (self.LA(1)==u'\\') and (self.LA(2)==u'}'):
                pass
                _saveIndex = self.text.length()
                self.match('\\')
                self.text.setLength(_saveIndex)
                self.match('}')
            elif (self.LA(1)==u'\\') and ((self.LA(2) >= u'\u0003' and self.LA(2) <= u'\ufffe')):
                pass
                self.mESC_CHAR(False, False)
            elif (self.LA(1)==u'{'):
                pass
                self.mNESTED_ANONYMOUS_TEMPLATE(False)
            elif (_tokenSet_4.member(self.LA(1))):
                pass
                self.matchNot('}')
            else:
                break
            
        self.match('}')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mLBRACK(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = LBRACK
        _saveIndex = 0
        pass
        self.match('[')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mRBRACK(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = RBRACK
        _saveIndex = 0
        pass
        self.match(']')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mLPAREN(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = LPAREN
        _saveIndex = 0
        pass
        self.match('(')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mRPAREN(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = RPAREN
        _saveIndex = 0
        pass
        self.match(')')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mCOMMA(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = COMMA
        _saveIndex = 0
        pass
        self.match(',')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mDOT(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = DOT
        _saveIndex = 0
        pass
        self.match('.')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mASSIGN(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = ASSIGN
        _saveIndex = 0
        pass
        self.match('=')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mCOLON(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = COLON
        _saveIndex = 0
        pass
        self.match(':')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mPLUS(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = PLUS
        _saveIndex = 0
        pass
        self.match('+')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mSEMI(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = SEMI
        _saveIndex = 0
        pass
        self.match(';')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mNOT(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = NOT
        _saveIndex = 0
        pass
        self.match('!')
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mDOTDOTDOT(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = DOTDOTDOT
        _saveIndex = 0
        pass
        self.match("...")
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    def mWS(self, _createToken):    
        _ttype = 0
        _token = None
        _begin = self.text.length()
        _ttype = WS
        _saveIndex = 0
        pass
        la1 = self.LA(1)
        if False:
            pass
        elif la1 and la1 in u' ':
            pass
            self.match(' ')
        elif la1 and la1 in u'\t':
            pass
            self.match('\t')
        elif la1 and la1 in u'\n\r':
            pass
            if (self.LA(1)==u'\r') and (self.LA(2)==u'\n'):
                pass
                self.match('\r')
                self.match('\n')
            elif (self.LA(1)==u'\r') and (True):
                pass
                self.match('\r')
            elif (self.LA(1)==u'\n'):
                pass
                self.match('\n')
            else:
                self.raise_NoViableAlt(self.LA(1))
            
            if not self.inputState.guessing:
                self.newline()
        else:
                self.raise_NoViableAlt(self.LA(1))
            
        if not self.inputState.guessing:
            _ttype = SKIP
        self.set_return_token(_createToken, _token, _ttype, _begin)
    
    

### generate bit set
def mk_tokenSet_0(): 
    data = [0L] * 2048 ### init list
    data[0] =-17179869192L
    data[1] =-268435457L
    for x in xrange(2, 1023):
        data[x] = -1L
    data[1023] =9223372036854775807L
    return data
_tokenSet_0 = antlr.BitSet(mk_tokenSet_0())

### generate bit set
def mk_tokenSet_1(): 
    data = [0L] * 1025 ### init list
    data[0] =4294977024L
    data[1] =576460745995190270L
    return data
_tokenSet_1 = antlr.BitSet(mk_tokenSet_1())

### generate bit set
def mk_tokenSet_2(): 
    data = [0L] * 1025 ### init list
    data[0] =288107235144377856L
    data[1] =1729382250602037246L
    return data
_tokenSet_2 = antlr.BitSet(mk_tokenSet_2())

### generate bit set
def mk_tokenSet_3(): 
    data = [0L] * 1025 ### init list
    data[0] =4294977024L
    return data
_tokenSet_3 = antlr.BitSet(mk_tokenSet_3())

### generate bit set
def mk_tokenSet_4(): 
    data = [0L] * 2048 ### init list
    data[0] =-8L
    data[1] =-2882303761785552897L
    for x in xrange(2, 1023):
        data[x] = -1L
    data[1023] =9223372036854775807L
    return data
_tokenSet_4 = antlr.BitSet(mk_tokenSet_4())

### generate bit set
def mk_tokenSet_5(): 
    data = [0L] * 1025 ### init list
    data[0] =17596481021440L
    return data
_tokenSet_5 = antlr.BitSet(mk_tokenSet_5())

### generate bit set
def mk_tokenSet_6(): 
    data = [0L] * 1025 ### init list
    data[0] =17596481021440L
    data[1] =576460745995190270L
    return data
_tokenSet_6 = antlr.BitSet(mk_tokenSet_6())
    
### __main__ header action >>> 
if __name__ == '__main__' :
    import sys
    import antlr
    import ActionLexer
    
    ### create lexer - shall read from stdin
    try:
        for token in ActionLexer.Lexer():
            print token
            
    except antlr.TokenStreamException, e:
        print "error: exception caught while lexing: ", e
### __main__ header action <<< 
